<?php

use gincore\Webhook;
use gincore\Settings;
use gincore\Api;
use gincore\Categories;
use gincore\Products;

class ControllerExtensionModuleGincore extends Controller
{

    /**
     * @var Webhook 
     */
    private $webhook;
    /**
     * @var Settings 
     */
    private $settings;
    
    /**
     * @param \Registry $registry
     */
    public function __construct($registry)
    {
        parent::__construct($registry);
        $this->settings = new Settings($registry);
    }
    
    /**
     * @return void
     */
    public function webhook()
    {
        if(!$this->settings->enabled()){
            http_response_code(406); // Not Acceptable
            echo 'Integration disabled';
            exit;
        }
        
        $this->webhook = new Webhook($this->settings->getSettings(), $this->registry);
        if(!$this->webhook->validateKey($this->request->get['key'])){
            http_response_code(403); // Forbidden
            echo 'Webhook key not valid';
            exit;
        }
        
        $this->api = new Api($this->settings->getSettings());
        
        $data = json_decode(file_get_contents('php://input'), true);
        if(isset($data['events'])){
            foreach($data['events'] as $event){
                $this->handleEvent($event);
            }
        }
    }
    
    /**
     * @param array $event
     * @return void
     */
    private function handleEvent($event)
    {
        foreach($event['data'] as $entity => $ids){
            $method = 'webhook' . ucfirst(strtolower($event['event'])) . ucfirst($entity);
            if(method_exists($this, $method)){
                $this->{$method}($ids);
            }
        }
    }
    
    /**
     * @param array $ids
     * @return void
     */
    private function webhookCreatedCategories($ids)
    {
        $data = $this->api->getFilteredCategories(['id' => implode(',', $ids)]);
        if(isset($data['categories'])){
            $categories = new Categories($this->settings->getSettings(), $this->registry);
            foreach($data['categories'] as $category){
                $categories->createCategory($category);
            }
            $categories->repairCategories();
        }
    }
    
    /**
     * @param array $ids
     * @return void
     */
    private function webhookUpdatedCategories($ids)
    {
        $data = $this->api->getFilteredCategories(['id' => implode(',', $ids)]);
        if(isset($data['categories'])){
            $categories = new Categories($this->settings->getSettings(), $this->registry);
            foreach($data['categories'] as $category){
                $categories->updateCategory($category);
            }
            $categories->repairCategories();
        }
    }
    
    /**
     * @param array $ids
     * @return void
     */
    private function webhookDeletedCategories($ids)
    {
        $categories = new Categories($this->settings->getSettings(), $this->registry);
        foreach($ids as $id){
            $categories->removeCategory($id);
        }
    }
    
    /**
     * @param array $ids
     * @return void
     */
    private function webhookCreatedProducts($ids)
    {
        $data = $this->api->getFilteredProducts(['id' => implode(',', $ids)]);
        if(isset($data['products'])){
            $products = new Products($this->settings->getSettings(), $this->registry);
            foreach($data['products'] as $product){
                $products->createProduct($product);
            }
        }
    }
    
    /**
     * @param array $ids
     * @return void
     */
    private function webhookUpdatedProducts($ids)
    {
        $data = $this->api->getFilteredProducts(['id' => implode(',', $ids)]);
        if(isset($data['products'])){
            $products = new Products($this->settings->getSettings(), $this->registry);
            foreach($data['products'] as $product){
                $products->updateProduct($product);
            }
        }
    }
    
    /**
     * @param array $ids
     * @return void
     */
    private function webhookDeletedProducts($ids)
    {
        $products = new Products($this->settings->getSettings(), $this->registry);
        foreach($ids as $id){
            $products->removeProduct($id);
        }
    }
    
    /**
     * @param type $route
     * @param array $args
     * @param type $output
     */
    public function customerLoggedIn($route, $args, $output)
    {
        if($this->cart->hasProducts()){
            return;
        }
        $this->load->model('account/customer');
        $customer_info = $this->model_account_customer->getCustomerByEmail($args[0]);
        if($customer_info){
            $api = new Api($this->settings->getSettings());
            $products = new Products($this->settings->getSettings(), $this->registry);
            
            $by_phone = $api->getUserCart('phone', $customer_info['telephone']);
            $by_email = $api->getUserCart('email', $customer_info['email']);
            
            $cart = isset($by_phone['cart']) ? $by_phone['cart'] : (isset($by_email['cart']) ? $by_email : []);
            foreach($cart as $product){
                $oc_product_id = $products->getOcProductId($product['product_id']);
                $oc_product_option = $products->getOcProductOption($product['product_id']);
                if(!$oc_product_id && !$oc_product_option){
                    continue;
                }
                $option = [];
                if($oc_product_option){
                    $oc_product_id = $oc_product_option['oc_product_id'];
                    $product_option = $this->db->query("SELECT product_option_id, product_option_value_id FROM " . DB_PREFIX . "product_option_value WHERE product_id = " . $oc_product_option['oc_product_id'] . " AND option_value_id = " . $oc_product_option['oc_option_value_id']);
                    if(!$product_option->rows){
                        continue;
                    }
                    $option = [$product_option->rows[0]['product_option_id'] => $product_option->rows[0]['product_option_value_id']];
                }
                $this->cart->add($oc_product_id, $product['qty'], $option);
            }
        }
    }

    /**
     * @param type $route
     * @param type $args
     * @param type $output
     */
    public function orderCreated($route, $args, $output) 
    {
        $this->load->language('extension/module/gincore');
        $this->load->model('checkout/order');
        $order_id = $args[0];
        $order_info = $this->model_checkout_order->getOrder($order_id);
        $order_products = $query = $this->db->query("SELECT * FROM " . DB_PREFIX . "order_product WHERE order_id = " . $order_id)->rows;
        $products = [];
        foreach($order_products as $order_product){
            $order_product_option = $query = $this->db->query("SELECT * FROM " . DB_PREFIX . "order_option WHERE order_id = " . $order_id . " AND order_product_id = " . $order_product['order_product_id'])->rows;
            if(!empty($order_product_option)){
                $q = $this->db->query("
                    SELECT gpo.gincore_option_product_id as gincore_product_id
                    FROM " . DB_PREFIX . "product_option_value as pov 
                    LEFT JOIN " . DB_PREFIX . "gincore_product_options as gpo ON gpo.oc_option_value_id = pov.option_value_id
                    WHERE pov.product_option_value_id = " . $order_product_option[0]['product_option_value_id'] . "
                ");
            }else{
                $q = $this->db->query("
                    SELECT gincore_product_id
                    FROM " . DB_PREFIX . "gincore_products 
                    WHERE oc_product_id = " . $order_product['product_id'] . "
                ");
            }
            if(empty($q->rows)){
               continue; 
            }
            $product_id = $q->rows[0]['gincore_product_id'];
            $products[] = [
                'product_id' => $product_id,
                'qty' => $order_product['quantity'],
                'price' => $order_product['price']
            ];
        }
        if(!$products){
            return;
        }
        
        $api = new Api($this->settings->getSettings());
        $result = $api->checkoutOrder([
            'products' => $products,
            'user' => [
                'phone' => $order_info['telephone'],
                'full_name' => $order_info['firstname'] . ' ' . $order_info['lastname']
            ],
            'delivery' => 3, // select from settings?
            'delivery_address' => implode(', ', array_filter([
                $order_info['shipping_address_1'],
                $order_info['shipping_address_2'],
                $order_info['shipping_postcode'],
                $order_info['shipping_city'],
                $order_info['shipping_zone'],
                $order_info['shipping_country']
            ])),
            'comment' => 
                ($order_info['comment'] ? $order_info['comment'] . "\n\n" : "")
                . $this->language->get('order_comment_shipping_method'). ": " . $order_info['shipping_method']
                . "\n" .  $this->language->get('order_comment_payment_method'). ": " . $order_info['payment_method']
        ]);
        
        $this->db->query("INSERT IGNORE INTO " . DB_PREFIX . "gincore_orders SET gincore_order_id = " . (isset($result['order']) ? $result['order']['id'] : 'null') . ", oc_order_id = " . $order_id . ", api_result = '" . json_encode($result) . "', created_at = NOW(), updated_at = NOW()");
    }
    
}
