<?php

namespace gincore;

class Api
{
    /**
     * @var array 
     */
    private $settings;
    
    /**
     * @param array $settings
     */
    public function __construct($settings)
    {
        $this->settings = $settings;
    }
    
    /**
     * Get category by filters
     * @param array $filters
     * @return array
     */
    public function getFilteredCategories($filters)
    {
        return $this->apiRequest('categories/filtered', $filters, 'POST');
    }
    
    /**
     * Get category by id
     * @param int $id
     * @return array
     */
    public function getCategory($id)
    {
        return $this->apiRequest('categories/' . $id);
    }
    
    /**
     * Get categories
     * @param int $parent_id = null
     * @param int $limit = 1000
     * @param int $offset = 0
     * @return array
     */
    public function getCategories($parent_id = null, $limit = 1000, $offset = 0)
    {
        $data = ['limit' => $limit, 'offset' => $offset];
        if(!is_null($parent_id)){
            $data['parent_id'] = $parent_id;
        }        
        return $this->apiRequest('categories', $data);
    }
    
    /**
     * Get categories from all pages
     * @param \Closure $callback
     * @param int $parent_id = null
     * @param int $limit = 1000
     * @param int $offset = 0
     * @return void
     */
    public function getCategoriesEach(\Closure $callback, $parent_id = null, $limit = 1000, $offset = 0)
    {
        $response = $this->getCategories($parent_id, $limit, $offset);
        $callback($response);
        if($this->hasMorePages($response)){
            $next_offset = $this->getNextPaginationOffset($response);
            $this->getCategoriesEach($callback, $parent_id, $limit, $next_offset);
        }
    }
    /**
     * Get products by filters
     * @param array $filters
     * @return array
     */
    public function getFilteredProducts($filters)
    {
        return $this->apiRequest('products/filtered', $filters, 'POST');
    }
    
    /**
     * Get product by id
     * @param int $id
     * @return array
     */
    public function getProduct($id)
    {
        return $this->apiRequest('products/' . $id);
    }
    
    /**
     * Get products
     * @param int $category_id = null
     * @param int $limit = 1000
     * @param int $offset = 0
     * @return array
     */
    public function getProducts($category_id = null, $limit = 1000, $offset = 0)
    {
        $data = ['limit' => $limit, 'offset' => $offset];
        if(!is_null($category_id)){
            $data['category_id'] = $category_id;
        }        
        return $this->apiRequest('products', $data);
    }
    
    /**
     * Get products from all pages
     * @param \Closure $callback
     * @param int $category_id = null
     * @param int $limit = 1000
     * @param int $offset = 0
     * @return void
     */
    public function getProductsEach(\Closure $callback, $category_id = null, $limit = 1000, $offset = 0)
    {
        $response = $this->getProducts($category_id, $limit, $offset);
        $callback($response);
        if($this->hasMorePages($response)){
            $next_offset = $this->getNextPaginationOffset($response);
            $this->getProductsEach($callback, $category_id, $limit, $next_offset);
        }
    }
    
    /**
     * @param array $response
     * @return int
     */
    public function getNextPaginationOffset($response)
    {
        if(!isset($response['pagination'])){
            return 0;
        }
        return $response['pagination']['offset'] + $response['pagination']['limit'];
    }
    
    /**
     * @param array $response
     * @return boolean
     */
    public function hasMorePages($response)
    {
        if(!isset($response['pagination'])){
            return false;
        }
        if($response['pagination']['offset'] + $response['pagination']['limit'] < $response['pagination']['total']){
            return true;
        }
        return false;
    }
    
    /**
     * @param array $order
     * @return array
     */
    public function checkoutOrder($order)
    {
        return $this->apiRequest('cart/checkout', $order, 'POST', ['Content-type: multipart/form-data']);
    }
    
    /**
     * @param array $order
     * @return array
     */
    public function getUserCart($field, $query)
    {
        return $this->apiRequest('cart', ['field' => $field, 'query' => $query]);
    }
    
    /**
     * @return boolean
     */
    public function apiTest()
    {
        $response = $this->apiRequest('cart/checkout-options'); // no special method for test 
        return !empty($response['options']);
    }
    
    /**
     * @param string $endpoint
     * @param array $data = []
     * @param string $method = GET
     * @param array $headers = []
     * @return array
     */
    public function apiRequest($endpoint, $data = [], $method = 'GET', $headers = [])
    {
        $url = 'https://' . $this->settings['api_domain'] . '/api/sites/' . $endpoint;
        
        $curl = curl_init();
        if($method == 'POST'){
            curl_setopt($curl, CURLOPT_POST, true);
            curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($data));
        }else{
            $url .= '?' . http_build_query($data);
        }
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 1);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_HTTPHEADER, [
            'Authorization: Bearer ' . $this->settings['api_key'],
            'Accept: application/json'
        ] + $headers);
        $response = curl_exec($curl);
        $code = curl_getinfo($curl, CURLINFO_RESPONSE_CODE);
        curl_close($curl);

        if(strpos((string)$code, '2') === 0){
            return json_decode($response, true);
        }
        return ['error' => 'Api not reachable', 'response' => $response, 'code' => $code];
    }

}
